import re

from memover import file_handler, media_file_extractor, language_codes

__subtitle_types = ('.srt', '.smi', '.ssa', '.ass', '.vtt')


def rename_and_move(source_directory):
    if not file_handler.path_is_directory(source_directory):
        return

    files = list(file_handler.get_files(source_directory))

    files.sort(key=lambda f: file_handler.get_last_path_part(f))

    for file_path in files:
        if __should_be_moved(file_path):
            file_path = __move_subtitle(file_path, source_directory)
        if __should_be_renamed(file_path):
            __rename(file_path)


def __should_be_moved(subtitle_path):
    if not subtitle_path.endswith(__subtitle_types):
        return False

    subtitle_dir = file_handler.get_parent(subtitle_path)
    biggest_file = file_handler.get_biggest_file(subtitle_dir)
    if media_file_extractor.is_media_file(biggest_file.path):
        return False

    return True


def __should_be_renamed(subtitle_path):
    if not subtitle_path.endswith(__subtitle_types):
        return False

    subtitle_dir = file_handler.get_parent(subtitle_path)
    biggest_file = file_handler.get_biggest_file(subtitle_dir)
    if not media_file_extractor.is_media_file(biggest_file.path):
        return False  # No media file. Nothing to rename to

    biggest_file_name = file_handler.get_last_path_part(biggest_file.path)
    subtitle_file_name = file_handler.get_last_path_part(subtitle_path)

    # should be moved if subtitle is not already named after media file in directory
    return subtitle_file_name.find(biggest_file_name) == -1


def __move_subtitle(subtitle_path, source_directory):
    for biggest_file in file_handler.get_biggest_files(file_handler.get_parent(subtitle_path), source_directory):
        if media_file_extractor.is_media_file(biggest_file.path):
            destination_path = file_handler.get_parent(biggest_file.path) + '/' + file_handler.get_last_path_part(subtitle_path)

            file_handler.move(
                subtitle_path,
                destination_path
            )
            __delete_empty_parents(subtitle_path, source_directory)
            return destination_path


def __rename(file_path):
    subtitle_type = file_handler.get_file_type(file_path)

    biggest_file = file_handler.get_biggest_file(file_handler.get_parent(file_path))
    if not media_file_extractor.is_media_file(biggest_file.path):
        return

    language_code = __identify_language(file_path)
    index = __calculate_index(language_code, file_path)

    destination_path = file_handler.get_path_without_extension(biggest_file.path) + '.' + language_code + index + subtitle_type

    file_handler.move(
        file_path,
        destination_path
    )


def __calculate_index(language_code, file_path):
    dir = file_handler.get_parent(file_path)
    files_in_dir = file_handler.get_directory_content(dir)
    files = [file for file in files_in_dir if re.search(r'\.' + language_code + r'\d*\.', file) is not None]
    index = len(files)
    return '' if index == 0 else str(index + 1)


def __identify_language(file_path):
    file = file_handler.get_last_path_part(file_path)
    by_code = language_codes.find_by_three_letter_code(file)
    if by_code:
        return by_code[1]

    by_language = language_codes.find_by_language(file)
    if by_language:
        return by_language[1]

    return 'en'  # Default to english if cant find a language in file name


def __delete_empty_parents(path, source_directory):
    current_parent = file_handler.get_parent(path)
    while current_parent != source_directory and file_handler.directory_is_empty(current_parent):
        file_handler.delete_directory(current_parent)
        current_parent = file_handler.get_parent(current_parent)


class NoMediaFileException(Exception):
    def __init__(self, message):
        super(NoMediaFileException, self).__init__(message)



import re

__CODES = (
    ('aar', 'aa', 'Afar'),
    ('abk', 'ab', 'Abkhazian'),
    ('afr', 'af', 'Afrikaans'),
    ('aka', 'ak', 'Akan'),
    ('amh', 'am', 'Amharic'),
    ('ara', 'ar', 'Arabic'),
    ('arg', 'an', 'Aragonese'),
    ('asm', 'as', 'Assamese'),
    ('ava', 'av', 'Avaric'),
    ('ave', 'ae', 'Avestan'),
    ('aym', 'ay', 'Aymara'),
    ('aze', 'az', 'Azerbaijani'),
    ('bak', 'ba', 'Bashkir'),
    ('bam', 'bm', 'Bambara'),
    ('bel', 'be', 'Belarusian'),
    ('ben', 'bn', 'Bengali'),
    ('bis', 'bi', 'Bislama'),
    ('bos', 'bs', 'Bosnian'),
    ('bre', 'br', 'Breton'),
    ('bul', 'bg', 'Bulgarian'),
    ('cha', 'ch', 'Chamorro'),
    ('che', 'ce', 'Chechen'),
    ('chv', 'cv', 'Chuvash'),
    ('cor', 'kw', 'Cornish'),
    ('cos', 'co', 'Corsican'),
    ('cre', 'cr', 'Cree'),
    ('dan', 'da', 'Danish'),
    ('dzo', 'dz', 'Dzongkha'),
    ('eng', 'en', 'English'),
    ('epo', 'eo', 'Esperanto'),
    ('est', 'et', 'Estonian'),
    ('ewe', 'ee', 'Ewe'),
    ('fao', 'fo', 'Faroese'),
    ('fij', 'fj', 'Fijian'),
    ('fin', 'fi', 'Finnish'),
    ('fry', 'fy', 'Western Frisian'),
    ('ful', 'ff', 'Fulah'),
    ('gle', 'ga', 'Irish'),
    ('glg', 'gl', 'Galician'),
    ('grn', 'gn', 'Guarani'),
    ('guj', 'gu', 'Gujarati'),
    ('hau', 'ha', 'Hausa'),
    ('heb', 'he', 'Hebrew'),
    ('her', 'hz', 'Herero'),
    ('hin', 'hi', 'Hindi'),
    ('hmo', 'ho', 'Hiri Motu'),
    ('hrv', 'hr', 'Croatian'),
    ('hun', 'hu', 'Hungarian'),
    ('ibo', 'ig', 'Igbo'),
    ('ido', 'io', 'Ido'),
    ('iku', 'iu', 'Inuktitut'),
    ('ind', 'id', 'Indonesian'),
    ('ipk', 'ik', 'Inupiaq'),
    ('ita', 'it', 'Italian'),
    ('jav', 'jv', 'Javanese'),
    ('jpn', 'ja', 'Japanese'),
    ('kan', 'kn', 'Kannada'),
    ('kas', 'ks', 'Kashmiri'),
    ('kau', 'kr', 'Kanuri'),
    ('kaz', 'kk', 'Kazakh'),
    ('khm', 'km', 'Central Khmer'),
    ('kin', 'rw', 'Kinyarwanda'),
    ('kom', 'kv', 'Komi'),
    ('kon', 'kg', 'Kongo'),
    ('kor', 'ko', 'Korean'),
    ('kur', 'ku', 'Kurdish'),
    ('lao', 'lo', 'Lao'),
    ('lat', 'la', 'Latin'),
    ('lav', 'lv', 'Latvian'),
    ('lin', 'ln', 'Lingala'),
    ('lit', 'lt', 'Lithuanian'),
    ('lub', 'lu', 'Luba-Katanga'),
    ('lug', 'lg', 'Ganda'),
    ('mah', 'mh', 'Marshallese'),
    ('mal', 'ml', 'Malayalam'),
    ('mar', 'mr', 'Marathi'),
    ('mlg', 'mg', 'Malagasy'),
    ('mlt', 'mt', 'Maltese'),
    ('mon', 'mn', 'Mongolian'),
    ('nau', 'na', 'Nauru'),
    ('ndo', 'ng', 'Ndonga'),
    ('nep', 'ne', 'Nepali'),
    ('nor', 'no', 'Norwegian'),
    ('oji', 'oj', 'Ojibwa'),
    ('ori', 'or', 'Oriya'),
    ('orm', 'om', 'Oromo'),
    ('pli', 'pi', 'Pali'),
    ('pol', 'pl', 'Polish'),
    ('por', 'pt', 'Portuguese'),
    ('que', 'qu', 'Quechua'),
    ('roh', 'rm', 'Romansh'),
    ('run', 'rn', 'Rundi'),
    ('rus', 'ru', 'Russian'),
    ('sag', 'sg', 'Sango'),
    ('san', 'sa', 'Sanskrit'),
    ('slv', 'sl', 'Slovenian'),
    ('sme', 'se', 'Northern Sami'),
    ('smo', 'sm', 'Samoan'),
    ('sna', 'sn', 'Shona'),
    ('snd', 'sd', 'Sindhi'),
    ('som', 'so', 'Somali'),
    ('sot', 'st', 'Sotho, Southern'),
    ('spa', 'es', 'Spanish'),
    ('srd', 'sc', 'Sardinian'),
    ('srp', 'sr', 'Serbian'),
    ('ssw', 'ss', 'Swati'),
    ('sun', 'su', 'Sundanese'),
    ('swa', 'sw', 'Swahili'),
    ('swe', 'sv', 'Swedish'),
    ('tah', 'ty', 'Tahitian'),
    ('tam', 'ta', 'Tamil'),
    ('tat', 'tt', 'Tatar'),
    ('tel', 'te', 'Telugu'),
    ('tgk', 'tg', 'Tajik'),
    ('tgl', 'tl', 'Tagalog'),
    ('tha', 'th', 'Thai'),
    ('tir', 'ti', 'Tigrinya'),
    ('tsn', 'tn', 'Tswana'),
    ('tso', 'ts', 'Tsonga'),
    ('tuk', 'tk', 'Turkmen'),
    ('tur', 'tr', 'Turkish'),
    ('twi', 'tw', 'Twi'),
    ('ukr', 'uk', 'Ukrainian'),
    ('urd', 'ur', 'Urdu'),
    ('uzb', 'uz', 'Uzbek'),
    ('ven', 've', 'Venda'),
    ('vie', 'vi', 'Vietnamese'),
    ('wln', 'wa', 'Walloon'),
    ('wol', 'wo', 'Wolof'),
    ('xho', 'xh', 'Xhosa'),
    ('yid', 'yi', 'Yiddish'),
    ('yor', 'yo', 'Yoruba'),
    ('zul', 'zu', 'Zulu')
)


def __find_by(file, element_position):
    def match_language(language):
        regex = r'(^|\.)' + language[element_position] + r'\.'
        matches = re.findall(regex, file)
        return len(matches) > 0

    return next((language for language in __CODES if match_language(language)), None)


def find_by_three_letter_code(file):
    return __find_by(file, 0)


def find_by_language(file):
    return __find_by(file, 2)



import unittest

from memover.arguments_parser import MeMoverArgs, Commands
from memover.watcher.path_queue import PathQueue


class TestMoverShows(unittest.TestCase):

    __SOURCE = 'inbox'

    def setUp(self):
        self.__args = MeMoverArgs(Commands.WATCH, self.__SOURCE, 'show-destination', 'movie-destinatino')
        self.__path_queue = PathQueue(self.__args)

    def test_append_dir(self):
        path = f'before-source/{self.__SOURCE}/after-source'
        self.__path_queue.append(path, 1000.0)
        stripped_path = f'{self.__SOURCE}/after-source'
        self.assertEqual(self.__path_queue, [stripped_path])
        self.__path_queue.append(f'{path}/test-file.mkv', 1000.0)
        self.assertEqual(self.__path_queue, [stripped_path])

    def test_append_file(self):
        path = f'before-source/{self.__args.source}/test.mkv'
        self.__path_queue.append(path, 1000.0)
        self.assertEqual([f'{self.__args.source}/test.mkv'], self.__path_queue)

    def test_in_queue_dir(self):
        path = f'before-source/{self.__SOURCE}/after-source'
        self.__path_queue.append(path, 1000.0)
        self.assertTrue(self.__path_queue.in_queue(path))

        # Should only look at root dir, adding not appended file in same dir
        self.assertTrue(self.__path_queue.in_queue(f'{path}/test-file.mkv'))

    def test_in_queue_file(self):
        path = f'before-source/{self.__args.source}/test.mkv'
        self.__path_queue.append(path, 1000.0)
        self.assertTrue(self.__path_queue.in_queue(f'{self.__SOURCE}/test.mkv'))

    def test_get_timestamp(self):
        path = f'before-source/{self.__SOURCE}/media-file-path'
        self.__path_queue.append(path, 1000.0)
        self.assertEqual(1000.0, self.__path_queue.get_last_modified(path))

    def test_get_timestamp_not_found(self):
        path = f'before-source/{self.__SOURCE}/media-file-path'
        self.assertIsNone(self.__path_queue.get_last_modified(path))

    def test_update_last_modified(self):
        path = f'before-source/{self.__SOURCE}/media-file-path'
        self.__path_queue.append(path, 1000.0)
        self.__path_queue.update_last_modified(path, 2000.0)
        self.assertEqual(2000.0, self.__path_queue.get_last_modified(path))

    def test_time_since_last_modify(self):
        path = f'before-source/{self.__SOURCE}/media-file-path'
        self.__path_queue.append(path, 1000.0)
        self.assertEqual(500.0, self.__path_queue.time_since_last_modified(path, 1500.0))

    def test_time_since_last_modify_not_found(self):
        path = f'before-source/{self.__SOURCE}/media-file-path'
        self.assertIsNone(self.__path_queue.time_since_last_modified(path, 1500.0))



import time
from dataclasses import dataclass
from typing import Optional

from memover import logger
from memover.arguments_parser import MeMoverArgs


@dataclass
class QueueItem:
    path: str
    last_modified: float
    modified: bool


class PathQueue:

    def __init__(self, args: MeMoverArgs):
        self.__queue = []
        self.__args = args

    def __eq__(self, other):
        if isinstance(other, list):
            return [item.path for item in self.__queue] == other
        return self is other

    def __repr__(self):
        return str([item.path for item in self.__queue])

    def is_empty(self):
        return not self.__queue

    def next(self) -> QueueItem:
        return self.__queue[0]

    def remove(self, path):
        cleaned_path = self.__clean_path(path)
        for item in self.__queue:
            if item.path == cleaned_path:
                self.__queue.remove(item)
                return

    def append(self, path, timestamp: float):
        cleaned_path = self.__clean_path(path)
        if self.in_queue(path):
            return
        self.__queue.append(QueueItem(path=cleaned_path, last_modified=timestamp, modified=True))

    def log_debug(self, queue_name):
        logger.debug(f'{queue_name}: {[item.path for item in self.__queue]}')

    def in_queue(self, path):
        cleaned_path = self.__clean_path(path)
        for item in self.__queue:
            if item.path == cleaned_path:
                return True
        return False

    def update_last_modified(self, path, timestamp: float):
        cleaned_path = self.__clean_path(path)
        for item in self.__queue:
            if item.path == cleaned_path:
                item.last_modified = timestamp
                item.modified = True
                return

    def get_last_modified(self, path) -> Optional[float]:
        cleaned_path = self.__clean_path(path)
        for item in self.__queue:
            if item.path == cleaned_path:
                return item.last_modified
        return None

    def time_since_last_modified(self, path, current_time: float = None) -> Optional[float]:
        if current_time is None:
            current_time = time.time()
        timestamp = self.get_last_modified(path)
        if timestamp is None:
            return None
        return current_time - timestamp

    def __clean_path(self, path):
        everything_after_source = self.__lstrip_source(path)
        everything_after_source = everything_after_source.lstrip('/')

        try:
            slash_index = everything_after_source.index('/')
            first_root_path = everything_after_source[:slash_index]
        except ValueError:
            first_root_path = everything_after_source

        first_root_path = first_root_path.rstrip('/')
        source = self.__args.source.rstrip('/')
        return f'{source}/{first_root_path}'

    def __lstrip_source(self, path):
        index = path.index(self.__args.source) + len(self.__args.source)
        return path[index:]



# coding=utf-8
import re

from . import file_handler
from . import show_name_extractor

__MEDIA_FILE_MIN_FILE_SIZE_MB = 50


class Type:
    MOVIE = 0
    TV_SHOW = 1


def is_media_file(file_path):
    return file_handler.get_file_size(file_path) >= __MEDIA_FILE_MIN_FILE_SIZE_MB


def get_type(path):
    if __path_contains_multiple_tv_episodes(path):
        return Type.TV_SHOW

    if __path_contains_at_least_one_episode(path):
        return Type.TV_SHOW

    return Type.MOVIE


def get_season(file_path):
    """
    :return: the original season number from file name
    """
    match = _get_episode_match(file_path)
    return match['season']


def get_season_number(file_path):
    """
    :return: the season number in form of an integer value
    """
    match = _get_episode_match(file_path)
    return int(match['season'])


def get_episode_number(file_path):
    """
        use with care. All episodes doe not have episode number
        :return: the episode number or None
    """
    match = _get_episode_match(file_path)
    return match['episode']


def get_tv_show_name(file_path):
    match = _get_episode_match(file_path)
    show_name = show_name_extractor.extract_show_name(match['show_name'])
    if show_name is None:
        parent_folder = file_handler.get_parent(file_path)
        parent_folder_name = file_handler.get_last_path_part(parent_folder)
        show_name = show_name_extractor.extract_show_name(parent_folder_name)
    if show_name is None:
        raise CouldNotExtractShowNameException(f'Could not extract show name from file path: {file_path}')
    return show_name


def get_file_type(file_path):
    return file_handler.get_file_type(file_path)


def episode_is_marked_proper(file_path):
    file_name = file_handler.get_last_path_part(file_path)
    return '.proper.' in file_name.lower()


def __path_contains_multiple_tv_episodes(path):
    episode_numbers = sorted(list(_iterate_episodes_numbers(path)))

    min_number_of_episodes = 5
    if len(episode_numbers) < min_number_of_episodes:
        return False

    for index, element in enumerate(episode_numbers[1:], start=1):
        # Allow same or one less
        if element is not episode_numbers[index - 1] + 1 and element is not episode_numbers[index - 1]:
            return False

    return True


def _iterate_episodes_numbers(path):
    for file_path in file_handler.get_files(path):
        if is_media_file(file_path) and __contains_episode_number(file_path):
            yield int(_get_episode_number_matches(file_path)[0])


def __contains_episode_number(file_path):
    matches = _get_episode_number_matches(file_path)
    return len(matches) > 0


def _get_episode_number_matches(file_path):
    file = '/' + file_handler.get_last_path_part(file_path)  # Add slash for regex compability
    matches = re.findall(r'(?<=\s|_|E|\/)\d+(?=\s|\w|\[)', file)
    return matches


def _get_show_name(file_path):
    local_path = file_handler.get_last_path_part(file_path)
    show_name = re.search(r'.*(?=(\s|_)\d+(\s|_))', local_path)
    if show_name:
        if show_name.group(0).lower() != 'episode':
            return show_name.group(0)
    parent_path = file_handler.get_parent(file_path)  # fall back on parent hoping it contains show name
    return file_handler.get_last_path_part(parent_path)


def __path_contains_at_least_one_episode(path):
    for file_path in file_handler.get_files(path):
        reg_tv_result = _get_episode_info(file_path)  # if we found episode info media_type is TV SHOW.
        if reg_tv_result is not None:
            return True
    return False


def _get_episode_info(path):
    match = __match_episode(path)

    # check if parent folder gives a episode match
    if match is None:
        parent_path = path.split('/')[-2]
        match = __match_episode(parent_path)

    return match


def __match_episode(path):
    """
    :param path: a path to something that is assumed to be a episode
    :return: extracted episode data (season, episode, show name)
    """
    file_basename = file_handler.get_last_path_part(path)
    reg_tv = re.compile('(.+?)[. ][Ss](\d\d?)[ .Ee]((\d\d?)?).*')
    match = reg_tv.match(file_basename)

    if match is None:
        reg_tv = re.compile(r'(.+)[\s?][-?][\s](\d\d?)[Xx](\d\d?)[\s](.+)')
        match = reg_tv.match(file_basename)

    return match


def _get_episode_match(file_path):
    reg_tv_result = _get_episode_info(file_path)

    if reg_tv_result is not None:
        show_name = reg_tv_result.group(1)
        season = reg_tv_result.group(2)
        episode = reg_tv_result.group(4)
    else:
        # fallback on just getting the episode number and assuming it's first season
        matches = _get_episode_number_matches(file_path)
        show_name = _get_show_name(file_path)

        if len(matches) == 0 or not show_name:
            raise WrongMediaTypeException('Can not parse episode')

        episode = matches[0]
        season = '01'

    return {
        'show_name': show_name,
        'season': season,
        'episode': episode
    }


class WrongMediaTypeException(Exception):
    def __init__(self, message):
        super(WrongMediaTypeException, self).__init__(message)


class CouldNotExtractShowNameException(Exception):
    def __init__(self, message):
        super(CouldNotExtractShowNameException, self).__init__(message)



import unittest
from memover import file_handler, file_matcher


class TestFileMatcher(unittest.TestCase):

    __SOURCE_DIRECTORY_PATH = 'testfolder'

    def setUp(self):
        file_handler.create_dir(self.__SOURCE_DIRECTORY_PATH)

    def tearDown(self):
        file_handler.delete_directory(self.__SOURCE_DIRECTORY_PATH)

    def runTest(self):
        self.__test_matching_file()
        self.__test_matching_file_in_folder()

    def __test_matching_file(self):
        file_path = self.__SOURCE_DIRECTORY_PATH + '/An.Other.Show.Info.S09E01.something.something-something[something].mp4'
        file_handler.create_file(file_path)

        search_for = 'An Other Show Info'
        file_paths = file_matcher.search_files(search_for, self.__SOURCE_DIRECTORY_PATH)
        self.assertEqual(1, len(file_paths))
        self.assertEqual(file_path, file_paths[0])

    def __test_matching_file_in_folder(self):
        folder_path = self.__SOURCE_DIRECTORY_PATH + '/hey.arnold.S09E01.SOMETHING.something-something'
        file_handler.create_dir(folder_path)
        search_for = 'hey arnold'

        file_paths = file_matcher.search_files(search_for, self.__SOURCE_DIRECTORY_PATH)
        self.assertEqual(1, len(file_paths))
        self.assertEqual(folder_path, file_paths[0])



import unittest
from memover import file_handler
from tests.utils import file_mover_tester


class TestFileHandler(unittest.TestCase, file_mover_tester.FileMoverTester):

    def setUp(self):
        self._create_test_dirs()

    def tearDown(self):
        self._delete_test_dirs()

    def test_get_path_without_extension(self):
        path = 'ColdCraft (2016) FinSub CDROMRip x264 KORIG'
        result = file_handler.get_path_without_extension(path)
        self.assertEqual(path, result)

    def test_get_biggest_file(self):
        file_path = self._createSourceFile('a/b/c/d/e/f/g.aa')
        directory_path = file_handler.get_parent(file_path)
        biggest_files = list(file_handler.get_biggest_files(directory_path, self._SOURCE_DIRECTORY))
        self.assertEqual(1, len(biggest_files))
        self.assertEqual(file_path, biggest_files[0].path)



import datetime
import re


def extract_show_name(show_name):
    show_name_words = __extract_urls(show_name) \
        .replace('.', ' ') \
        .split()

    show_name_words = __remove_release_date(show_name_words)
    show_name_words = __remove_sample_word(show_name_words)
    show_name_words = __remove_garbage_words(show_name_words)
    show_name_words = __remove_meta_info(show_name_words)
    show_name_words = __trim_garbage_chars(show_name_words)
    show_name_words = __strip_from_season(show_name_words)
    extracted_show_name = ' '.join(show_name_words)
    if extracted_show_name == '':
        return None
    return extracted_show_name


def __extract_urls(show_name):
    if 'www.' not in show_name.lower():
        return show_name

    url_start_index = show_name.find('www.')
    url_end_index = show_name.find(' ', url_start_index)
    show_name = show_name.replace(show_name[url_start_index:url_end_index], '')
    return __extract_urls(show_name)


def __remove_release_date(show_name_words):
    d1 = datetime.date(2000, 1, 1)
    d2 = datetime.date.today()

    try:
        last_word_date = datetime.datetime.strptime(show_name_words[-1], '%Y').date()
        if d1 <= last_word_date <= d2:
            del show_name_words[-1]
    except (ValueError, IndexError):
        pass
    return show_name_words


def __remove_sample_word(show_name_words):
    return [re.sub(r'\W*sample\W*', '', word, count=0, flags=re.IGNORECASE) for word in show_name_words]


def __remove_garbage_words(show_name_words):
    garbage_words = ['title']
    lower_cases = list(map(str.lower, show_name_words))

    for garbage_word in garbage_words:
        for word in lower_cases:
            if word == garbage_word:
                index = lower_cases.index(garbage_word)
                del show_name_words[index]
                del lower_cases[index]
    return show_name_words


def __remove_meta_info(show_name_words):
    if '[' in show_name_words and ']' in show_name_words:
        start_meta_info = show_name_words.index('[')
        end_meta_info = show_name_words.index(']')
        del show_name_words[start_meta_info:end_meta_info + 1]
        return __remove_meta_info(show_name_words)

    return show_name_words


def __trim_garbage_chars(show_name_words):
    if not show_name_words:
        return show_name_words

    garbage_chars = ['-']

    if show_name_words[0] in garbage_chars:
        show_name_words.remove(show_name_words[0])
        return __trim_garbage_chars(show_name_words)

    if show_name_words[-1] in garbage_chars:
        show_name_words.remove(show_name_words[-1])
        return __remove_meta_info(show_name_words)

    return show_name_words


def __strip_from_season(show_name_words):
    lower_cases = list(map(str.lower, show_name_words))
    if 'season' in lower_cases:
        strip_from = lower_cases.index('season')
        del show_name_words[strip_from:]
    return show_name_words



import unittest

from memover import file_handler, mover
from tests.utils import file_mover_tester


class TestMoverMovies(unittest.TestCase, file_mover_tester.FileMoverTester):

    def setUp(self):
        self._create_test_dirs()

    def tearDown(self):
        self._delete_test_dirs()

    def test_move_movie_by_file_path(self):
        movie_file_name = 'Abcde Fgh ijk Wjere To Found In 2010 HD-AA x111-ABC.mkv'
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + movie_file_name)

        source_file_path = self._SOURCE_DIRECTORY + '/' + movie_file_name
        mover.move_media_by_path(source_file_path, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)

        destination_path = self._MOVIE_DESTINATION_DIRECTORY + '/' + file_handler.get_last_name_from_path(movie_file_name) + '/' + movie_file_name
        self._assert_file_moved(source_file_path, destination_path)

    def test_moving_movie_in_directory(self):
        folder_path = self._SOURCE_DIRECTORY + '007 Going For Old Time HD-TS x264-CPG'
        file_handler.create_dir(folder_path)
        file_path = folder_path + '/007 Going For Old Time HD-TS x264-CPG.mp4'
        file_handler.create_file(file_path)
        mover.move_media_by_path(folder_path, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)
        file_destination_path = self._MOVIE_DESTINATION_DIRECTORY + '007 Going For Old Time HD-TS x264-CPG/007 Going For Old Time HD-TS x264-CPG.mp4'
        self._assert_file_moved(file_path, file_destination_path)

    def test_moving_movie_with_image(self):
        folder_path = self._SOURCE_DIRECTORY + 'Konstig (2017) [1080p] [JKL.KL]'
        file_handler.create_dir(folder_path)
        source_file_path = folder_path + '/WWW.YTS.AG.jpg'
        file_handler.create_file(source_file_path)

        mover.move_media_by_path(folder_path, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)

        file_destination_path = self._MOVIE_DESTINATION_DIRECTORY + '/Konstig (2017) [1080p] [JKL.KL]/WWW.YTS.AG.jpg'
        self._assert_file_moved(source_file_path, file_destination_path)

    def test_moving_movies_by_name(self):
        file_1_folder_path = self._SOURCE_DIRECTORY + '/201 Coolings 1999.DVDRIP.Xvid.NVesub-'
        file_handler.create_dir(file_1_folder_path)
        file_1_path = file_1_folder_path + '/201 Coolings 1999.___RIP.Xvid.NVesub-123.mp4'
        file_handler.create_file(file_1_path)

        file_2_path = self._SOURCE_DIRECTORY + '/201 Coolings 200.DADRAP.NVesub.mp4'
        file_handler.create_file(file_2_path)

        mover.move_media_by_name(
            '201 Coolings',
            self._SOURCE_DIRECTORY,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        file_destination_path = self._MOVIE_DESTINATION_DIRECTORY + '/201 Coolings 1999.DVDRIP.Xvid.NVesub-/201 Coolings 1999.___RIP.Xvid.NVesub-123.mp4'
        self._assert_file_moved(file_1_folder_path, file_destination_path)

        file_destination_path = self._MOVIE_DESTINATION_DIRECTORY + '/201 Coolings 200.DADRAP.NVesub/201 Coolings 200.DADRAP.NVesub.mp4'
        self._assert_file_moved(file_2_path, file_destination_path)

    def test_moving_movie_with_extra_dir(self):
        movie_dir = 'Movie.The.Movie.1234'
        file_handler.create_dir(self._SOURCE_DIRECTORY + movie_dir)

        movie_file = movie_dir + '/Movie.The.Movie.1234.mkv'
        file_handler.create_file(self._SOURCE_DIRECTORY + movie_file)

        extras_dir = movie_dir + '/Extras'
        file_handler.create_dir(self._SOURCE_DIRECTORY + extras_dir)

        extras_file1 = extras_dir + '/Movie.The.Movie.1234.Theatrical.Trailer.mkv'
        file_handler.create_file(self._SOURCE_DIRECTORY + extras_file1)

        extras_file2 = extras_dir + '/Movie.3.Theatrical.Trailer.mkv'
        file_handler.create_file(self._SOURCE_DIRECTORY + extras_file2)

        extras_soundtrack_dir = extras_dir + '/Soundtrack Live'
        file_handler.create_dir(self._SOURCE_DIRECTORY + extras_soundtrack_dir)

        extras_soundtrack_file1 = extras_soundtrack_dir + '/Making.the.Soundtrack.mkv'
        file_handler.create_file(self._SOURCE_DIRECTORY + extras_soundtrack_file1)

        extras_soundtrack_file2 = extras_soundtrack_dir + '/original.song.mkv'
        file_handler.create_file(self._SOURCE_DIRECTORY + extras_soundtrack_file2)

        mover.move_media_by_path(
            self._SOURCE_DIRECTORY + movie_dir,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        self._assert_file_moved(movie_file, self._MOVIE_DESTINATION_DIRECTORY + movie_file)
        self._assert_file_moved(extras_file1, self._MOVIE_DESTINATION_DIRECTORY + extras_file1)
        self._assert_file_moved(extras_file2, self._MOVIE_DESTINATION_DIRECTORY + extras_file2)
        self._assert_file_moved(extras_soundtrack_file1, self._MOVIE_DESTINATION_DIRECTORY + extras_soundtrack_file1)
        self._assert_file_moved(extras_soundtrack_file2, self._MOVIE_DESTINATION_DIRECTORY + extras_soundtrack_file2)

    def test_movie_destination_no_slash_in_path(self):
        dir_name = 'Tah Felling Out Of hope In Valley (2019) [Converted] [movie-test-no-path]/'
        file_handler.create_dir(self._SOURCE_DIRECTORY + dir_name)
        movie_file = 'Tah.Fellingr Out Of hope In Valley.2019.1080p.Converted-[movie-test-no-path].mp4'
        file_handler.create_file(self._SOURCE_DIRECTORY + dir_name + movie_file)

        mover.move_media_by_path(
            self._SOURCE_DIRECTORY + dir_name,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY[:-1]
        )

        self._assert_file_moved(
            self._SOURCE_DIRECTORY + dir_name + movie_file,
            self._MOVIE_DESTINATION_DIRECTORY + dir_name + movie_file
        )



import logging
import unittest

from memover.arguments_parser import MeMoverArgs, Commands
from memover.watcher.synced_watcher import SyncedWatcher
from tests.utils import file_mover_tester


class TestSyncedWatcher(unittest.TestCase, file_mover_tester.FileMoverTester):

    __episode_name = 'kolla.S04E15.asswe.xTTT-RR[abf]'
    __synced_watcher = None

    def setUp(self):
        self._create_test_dirs()
        args = MeMoverArgs(
            Commands.WATCH,
            self._SOURCE_DIRECTORY,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY,
            log_level=logging.DEBUG
        )
        self.__synced_watcher = SyncedWatcher(args, stable_seconds=0)

    def tearDown(self):
        self._delete_test_dirs()

    def test_dont_add_none_existing_file(self):
        none_existing_file_path = f'{self._SOURCE_DIRECTORY}{self.__episode_name}'
        self.__synced_watcher.on_created(none_existing_file_path)
        self.assertEqual(self.__synced_watcher.created_paths, [])

    def test_only_root_added_queues(self):
        # Create dir
        dir_1 = self._create_source_dir(self.__episode_name)
        self.__synced_watcher.on_created(dir_1)

        # Assert dir in both queues
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Create file 1
        file_1_data = self.__create_file_and_trigger_created('DO_NOT_MIRROR.exe')

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Modify dir
        self.__synced_watcher.on_modified(dir_1)

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Create file 2
        file_2_data = self.__create_file_and_trigger_created(f'{self.__episode_name}-mytv.mkv')

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Modify dir
        self.__synced_watcher.on_modified(dir_1)

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Create file 3
        file_3_data = self.__create_file_and_trigger_created('Gamma.txt')

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Modify dir
        self.__synced_watcher.on_modified(dir_1)

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Modify file 2
        self.__synced_watcher.on_modified(file_2_data['path'])
        self.__synced_watcher.on_modified(file_2_data['path'])
        self.__synced_watcher.on_modified(file_2_data['path'])

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Try to move
        self.__synced_watcher.move_next_path()

        # Assert dir in created_paths
        self.assertEqual(self.__synced_watcher.created_paths, [dir_1])

        # Actually moving
        self.__synced_watcher.move_next_path()

        # Assert Moved
        self.assert_moved(file_3_data)
        self.assert_moved(file_2_data)
        self.assert_moved(file_1_data)

        # Assert queues cleared
        self.assertEqual(self.__synced_watcher.created_paths, [])

    def __create_file_and_trigger_created(self, file):
        relative_path = f'{self.__episode_name}/{file}'
        path = self._createSourceFile(relative_path)
        self.__synced_watcher.on_created(path)
        return {
            'relative_path': relative_path,
            'path': path,
            'file': file
        }

    def assert_moved(self, file_data):
        self._assert_file_moved(
            file_data['relative_path'],
            f'{self._SHOW_DESTINATION_DIRECTORY}/kolla/Season 4/{file_data["file"]}'
        )



import asyncio

from memover.watcher import async_watcher
from . import arguments_parser, logger
from . import mover


def main():
    args = arguments_parser.get_args()
    if not args:
        return

    logger.setup(args.log_level)

    if args.type == arguments_parser.Commands.BY_NAME:
        mover.move_media_by_name(
            args.media_name,
            args.source,
            args.show_destination,
            args.movie_destination
        )
        return

    if args.type == arguments_parser.Commands.BY_PATH:
        mover.move_media_by_path(
            args.source,
            args.show_destination,
            args.movie_destination
        )
        return

    if args.type == arguments_parser.Commands.WATCH:
        asyncio.run(async_watcher.run(args))
        return



import os
import subprocess
import sys
import unittest

from tests.utils import file_mover_tester


class TestCliInterface(unittest.TestCase, file_mover_tester.FileMoverTester):

    def setUp(self):
        self._create_test_dirs()

    def tearDown(self):
        self._delete_test_dirs()

    def test_watch(self):
        args = 'watch test-source-watch test-show-destination-path test-movie-destination'
        result = execute(args)
        self.assertEqual(b'type: Commands.WATCH, source: test-source-watch, show_destination: test-show-destination-path, movie_destination: test-movie-destination, moves_before_quit: None, media_name: None\n', result)

    def test_watch_quit_after_moves(self):
        args = 'watch test-source-watch test-show-destination-path test-movie-destination --moves 4'
        result = execute(args)
        self.assertEqual(b'type: Commands.WATCH, source: test-source-watch, show_destination: test-show-destination-path, movie_destination: test-movie-destination, moves_before_quit: 4, media_name: None\n', result)

    def test_by_name(self):
        args = 'by-name "The name of my show " test-source-dir test-show-destination test-movie-destination'
        result = execute(args)
        self.assertEqual(b'type: Commands.BY_NAME, source: test-source-dir, show_destination: test-show-destination, movie_destination: test-movie-destination, moves_before_quit: None, media_name: The name of my show \n', result)

    def test_by_path(self):
        args = 'by-path test-source-path test-show-destination test-movie-destination'
        result = execute(args)
        self.assertEqual(b'type: Commands.BY_PATH, source: test-source-path, show_destination: test-show-destination, movie_destination: test-movie-destination, moves_before_quit: None, media_name: None\n', result)


def execute(args):
    p = subprocess.Popen(f'{sys.executable} {os.path.dirname(os.path.realpath(__file__))}/../memover/arguments_parser.py {args}', shell=True, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    output = p.communicate()
    result = output[0]
    print(result)
    return result



import re

from . import file_handler
from . import file_matcher
from . import logger
from . import media_file_extractor


def move(root_destination, path):
    failed_moved_files = []
    current_show_destination_path = None

    for file_path in file_handler.get_files(path):
        try:
            current_show_destination_path = __move_file_to_show_destination(
                root_destination,
                file_path
            )

        except media_file_extractor.WrongMediaTypeException:
            failed_moved_files.append(file_path)

    # move all failed files to root of show path
    if current_show_destination_path:
        for failed_moved_file_path in failed_moved_files:
            file_handler.move(
                failed_moved_file_path,
                current_show_destination_path + file_handler.get_last_path_part(failed_moved_file_path))


def __move_file_to_show_destination(root_destination, file_path):
    show_name_dir_name = __find_show_name_dir(root_destination, media_file_extractor.get_tv_show_name(file_path))
    if not show_name_dir_name:
        show_name_dir_name = __create_show_dir(root_destination, media_file_extractor.get_tv_show_name(file_path))

    season_number = str(media_file_extractor.get_season_number(file_path))
    season_path = root_destination + '/' + show_name_dir_name + '/Season ' + season_number
    if file_handler.directory_exist(season_path):
        __remove_old_if_new_is_proper(file_path, season_path)
    else:
        __create_season_folder(root_destination, show_name_dir_name, season_number)

    file_handler.move(file_path, season_path + '/' + file_handler.get_last_path_part(file_path))
    logger.info('"' + file_path + '" moved to: "' + season_path + '"')
    return root_destination + '/' + show_name_dir_name + '/'


def __remove_old_if_new_is_proper(file_path, season_dir_path):
    if not media_file_extractor.episode_is_marked_proper(file_path):
        return

    search_query = media_file_extractor.get_tv_show_name(file_path) + ' S' + media_file_extractor.get_season(file_path) + ' E' + media_file_extractor.get_episode_number(file_path)
    files = file_matcher.search_files_with_file_type(search_query, season_dir_path, media_file_extractor.get_file_type(file_path))
    if len(files) == 0:
        return

    for found_file in files:
        file_handler.delete_file(found_file)


def __create_show_dir(root_destination, show_name):
    show_destination = root_destination + '/' + show_name
    logger.debug('Directory does not exist. Creating: ' + show_destination)
    file_handler.create_dir(show_destination)
    return show_name


def __create_season_folder(root_destination, show_name, season_number):
    season_path = root_destination + '/' + show_name + '/Season ' + season_number
    logger.debug('Directory does not exist. Creating: ' + season_path)
    file_handler.create_dir(season_path)


def __find_show_name_dir(root_directory, searching_show_name):
    search_query = searching_show_name.lower().strip()

    logger.debug('Searching for matching directories with query: "' + search_query + '".')

    found_directories = __find_matching_directories(root_directory, search_query)
    if len(found_directories) == 1:
        return found_directories[0]

    # remove words with one or two letters in the beginning of the name if number of characters are bigger then five
    if len(search_query) > 5 and len(search_query.split()) >= 2:
        shortword_regex = re.compile(r'^\w{1,2}\b|\b\w{1,2}$')
        search_query = shortword_regex.sub('', search_query)

    found_directories = __find_matching_directories(root_directory, search_query)

    if len(found_directories) > 1:
        raise MultipleDirectoryMatchesException(searching_show_name, root_directory, found_directories)

    if len(found_directories) == 0 and 'proper' in search_query:
        return __find_show_name_dir(
            root_directory,
            search_query.replace('proper', '').strip()
        )  # remove proper key word, trim string and try again

    if len(found_directories) == 0:
        return None

    return found_directories[0]


def __is_right_season_directory(path, season_number):
    season_name = 'Season ' + str(season_number)
    if season_name in path:
        return True
    return False


def __find_matching_directories(root_directory, search_query):
    return [dir_name for dir_name in file_handler.get_directory_content(root_directory) if search_query in dir_name.lower().strip()]


class MultipleDirectoryMatchesException(Exception):

    def __init__(self, show_name, destination_folder, matching_dirs):
        matches = ', '.join(matching_dirs)
        super(MultipleDirectoryMatchesException, self).__init__(
            f'{show_name} matches multiple directories when moved {destination_folder}. Matches: {matches}'
        )



import unittest
from memover import file_handler, subtitles, mover
from tests.utils import file_mover_tester


class TestSubtitles(unittest.TestCase, file_mover_tester.FileMoverTester):

    def setUp(self):
        self._create_test_dirs()

    def tearDown(self):
        self._delete_test_dirs()

    def test_renaming_subs(self):
        source_files = [
            'a_movie_with_subs/Another.Another.S02E03.bigfile.mp4',
            'a_movie_with_subs/intetest/Other.Other.Other.Other.S09E01.something.something-something[something].swe.srt',
            'a_movie_with_subs/Other.Other.Other.Other.S09E01.something.something-something[something].smi',
            'a_movie_with_subs/[ www.flobbing.com ] - /Other.Other.Other.Other.S09E01.something.something-something[something].ssa',
            'a_movie_with_subs/[ www.flobbing.com ] - Longer.S01E02.HaHV.Maam-A/Information Downloaded From www.akkero.com.ass',
            'a_movie_with_subs/a/b/www.Lobertar.com - Long.S01E02.720p.KOOK.asdf-Risig/Stuff from www.Lobertar.com.vtt',
            'a_movie_with_subs/www.Lobertar.com - Long.S01E02.720p.KOOK.asdf-Risig/Stuff from www.Lobertar.com.txt',
            'a_movie_with_subs/intetest/Other.Other.Other.Other.S09E01.something.something-something[something].eng.srt'
        ]

        subtitles_file = [
            'a_movie_with_subs/Another.Another.S02E03.bigfile.en3.smi',
            'a_movie_with_subs/Another.Another.S02E03.bigfile.en.ass',
            'a_movie_with_subs/Another.Another.S02E03.bigfile.en5.vtt',
            'a_movie_with_subs/Another.Another.S02E03.bigfile.en4.ssa',
            'a_movie_with_subs/Another.Another.S02E03.bigfile.en2.srt',
            'a_movie_with_subs/Another.Another.S02E03.bigfile.sv.srt',
        ]

        # Create source files
        for source_file in source_files:
            self._createSourceFile(source_file)

        # make the first file the biggest
        self._set_size_in_mb(source_files[0], 60)

        subtitles.rename_and_move(self._SOURCE_DIRECTORY)

        for path in subtitles_file:
            self.assertTrue(file_handler.file_exist(self._SOURCE_DIRECTORY + path))

        #  assert that none sub files have not changed
        self.assertTrue(file_handler.file_exist(self._SOURCE_DIRECTORY + source_files[0]))

        self.assertFalse(
            file_handler.directory_exist(file_handler.get_parent(self._SOURCE_DIRECTORY + source_files[1]))
        )

        self.assertFalse(
            file_handler.file_exist(self._SOURCE_DIRECTORY + source_files[2])
        )

        self.assertFalse(
            file_handler.directory_exist(file_handler.get_parent(self._SOURCE_DIRECTORY + source_files[3]))
        )

        self.assertFalse(
            file_handler.file_exist(file_handler.get_parent(self._SOURCE_DIRECTORY + source_files[4]))
        )

        self.assertFalse(
            file_handler.directory_exist('testing/1')
        )

        self.assertTrue(file_handler.file_exist(self._SOURCE_DIRECTORY + source_files[6]))

    def test_moving_movie_with_sub_dir(self):
        class Destinations:

            def __init__(self, source, destination=None):
                self.source = source
                self.destination = destination if destination is not None else self.source

        movie_path = 'Castles.at.Sky.ghj'

        paths = [
            Destinations(movie_path + '/Castles.at.Sky.ghj.mp4'),
            Destinations(movie_path + '/Castles.at.Sky.ghj.nfo'),
            Destinations(movie_path + '/subs/English.srt', movie_path + '/Castles.at.Sky.ghj.en.srt'),
            Destinations(movie_path + '/subs/Spanish.srt', movie_path + '/Castles.at.Sky.ghj.es.srt'),
            Destinations(movie_path + '/subs/Portuguese.srt', movie_path + '/Castles.at.Sky.ghj.pt.srt'),
            Destinations(movie_path + '/subs/Swedish.srt', movie_path + '/Castles.at.Sky.ghj.sv.srt'),
            Destinations(movie_path + '/subs/Norwegian.srt', movie_path + '/Castles.at.Sky.ghj.no.srt'),
            Destinations(movie_path + '/subs/Danish.srt', movie_path + '/Castles.at.Sky.ghj.da.srt'),
            Destinations(movie_path + '/subs/Finnish.srt', movie_path + '/Castles.at.Sky.ghj.fi.srt'),
            Destinations(movie_path + '/subs/Indonesian.srt', movie_path + '/Castles.at.Sky.ghj.id.srt'),
            Destinations(movie_path + '/subs/Spanish.srt', movie_path + '/Castles.at.Sky.ghj.id.srt'),
            Destinations(movie_path + '/something_else.paa')
        ]

        for path in paths:
            self._createSourceFile(path.source)

        self._set_size_in_mb(paths[0].source, 60)

        mover.move_media_by_path(
            self._SOURCE_DIRECTORY + movie_path,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        for path in paths:
            self._assert_file_moved(path.source, self._MOVIE_DESTINATION_DIRECTORY + path.destination)



import unittest
from tests.utils import file_mover_tester
from memover import file_handler, episode_mover


class TestEpisodeMover(unittest.TestCase, file_mover_tester.FileMoverTester):

    def setUp(self):
        self._create_test_dirs()
        self.__TV_SHOW_FILE_SOURCE_PATH = 'aaa.bbb.s01e04.something.something-something.mp4'
        self.__TV_SHOW_FILE_DESTINATION_PATH = self._SHOW_DESTINATION_DIRECTORY + 'aaa bbb/Season 1/aaa.bbb.s01e04.something.something-something.mp4'

        self.__TV_SHOW_2_FILE_1_SOURCE_PATH = 'Moradeaa.S01E03.something.something-something.mkv'
        self.__TV_SHOW_2_FILE_1_DESTINATION_PATH = self._SHOW_DESTINATION_DIRECTORY + 'Moradeaa/Season 1/Moradeaa.S01E03.something.something-something.mkv'

        self.__TV_SHOW_2_FILE_2_SOURCE_PATH = 'Moradeaa.S01E02.something.something-something.mkv'
        self.__TV_SHOW_2_FILE_2_DESTINATION_PATH = self._SHOW_DESTINATION_DIRECTORY + 'Moradeaa/Season 1/Moradeaa.S01E02.something.something-something.mkv'

        self.__TV_SHOW_2_FILE_3_ORIGINAL_2 = 'Mordeaa.S06E14.720p.HDTV.x000-Argos.mkv'
        self.__TV_SHOW_2_FILE_ORIGINAL_2_NFO = 'Mordeaa.S06E14.720p.HDTV.x000-Argos.nfo'
        self.__TV_SHOW_2_FILE_ORIGINAL_1 = 'Mordeaa.S06E15.720p.HDTV.x264-Sallad[rarbgag].mkv'
        self.__TV_SHOW_2_FILE_ORIGINAL_1_NFO = 'Mordeaa.S06E15.720p.HDTV.x264-Sallad[rarbgag].nfo'
        self.__TV_SHOW_2_FILE_PROPER = 'Mordeaa.S06E15.PROPER.720p.HDTV.x264-ARGON-[123].mkv'
        self.__TV_SHOW_2_FILE_PROPER_NFO = 'Mordeaa.S06E15.PROPER.720p.HDTV.x264-ARGON-[123].nfo'

        self.__TV_SHOW_3_SEASON_DESTINATION_DIRECTORY_PATH = self._SHOW_DESTINATION_DIRECTORY + 'ABC/Season 9'
        self.__TV_SHOW_3_FILE_SOURCE_PATH = 'abc.S09E02.something.something-something.mp4'
        self.__TV_SHOW_3_FILE_DESTINATION_PATH = self.__TV_SHOW_3_SEASON_DESTINATION_DIRECTORY_PATH + '/abc.S09E02.something.something-something.mp4'

        self.__TV_SHOW_4_DESTINATION_DIRECTORY_PATH = self._SHOW_DESTINATION_DIRECTORY + 'Ab Cd Ef Gh/Season 10'
        self.__TV_SHOW_4_SOURCE_PATH = 'Ab.Cd.Ef.Gh.PROPER.S10E13.720p.HDTV.q123-FLOOR.mkv'
        self.__TV_SHOW_4_DESTINATION_PATH = self.__TV_SHOW_4_DESTINATION_DIRECTORY_PATH + '/Ab.Cd.Ef.Gh.PROPER.S10E13.720p.HDTV.q123-FLOOR.mkv'

        self.__TV_SHOW_5_FILE_SOURCE_PATH = 'Kongrass.2016.S01E06.720p.KONG-GG.maker.MAK.SWE.mkv'
        self.__TV_SHOW_5_DESTINATION_PATH = self._SHOW_DESTINATION_DIRECTORY + 'Kongrass/Season 1/Kongrass.2016.S01E06.720p.KONG-GG.maker.MAK.SWE.mkv'

        self.__SAMPLE_FILE_SOURCE_DIRECTORY = 'Felkod.S01E03.asd.dsa.dsa-AJKL[rarbg/Sample/'

        self.__SAMPLE_FILE_1_SOURCE_PATH = self.__SAMPLE_FILE_SOURCE_DIRECTORY + 'sample-felkod.s01e03.720p.ui.qq-oiu.mkv'
        self.__SAMPLE_FILE_1_DESTINATION_PATH = self._SHOW_DESTINATION_DIRECTORY + 'felkod/Season 1/sample-felkod.s01e03.720p.ui.qq-oiu.mkv'

        self.__SAMPLE_FILE_2_SOURCE_PATH = self.__SAMPLE_FILE_SOURCE_DIRECTORY + 'sample.felkod.s01e03.720p.ui.qq-oiu.mkv'
        self.__SAMPLE_FILE_2_DESTINATION_PATH = self._SHOW_DESTINATION_DIRECTORY + 'felkod/Season 1/sample.felkod.s01e03.720p.ui.qq-oiu.mkv'

        self.__SAMPLE_FILE_3_SOURCE_PATH = self.__SAMPLE_FILE_SOURCE_DIRECTORY + 'SaMple-felkod.s01e03.720p.aaa.x123-RORING.mkv'
        self.__SAMPLE_FILE_3_DESTINATION_PATH = self._SHOW_DESTINATION_DIRECTORY + 'felkod/Season 1/SaMple-felkod.s01e03.720p.aaa.x123-RORING.mkv'

        self._createSourceFile(self.__TV_SHOW_2_FILE_1_SOURCE_PATH)
        self._createSourceFile(self.__TV_SHOW_FILE_SOURCE_PATH)
        self._createSourceFile(self.__TV_SHOW_2_FILE_2_SOURCE_PATH)
        self._createSourceFile(self.__TV_SHOW_3_FILE_SOURCE_PATH)
        self._createSourceFile(self.__TV_SHOW_4_SOURCE_PATH)

        file_handler.create_dir(self.__TV_SHOW_3_SEASON_DESTINATION_DIRECTORY_PATH)
        file_handler.create_dir(self.__TV_SHOW_4_DESTINATION_DIRECTORY_PATH)

        file_handler.create_file(self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_ORIGINAL_1)
        file_handler.create_file(self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_ORIGINAL_1_NFO)
        file_handler.create_file(self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_3_ORIGINAL_2)
        file_handler.create_file(self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_ORIGINAL_2_NFO)
        self._createSourceFile(self.__TV_SHOW_2_FILE_PROPER)
        self._createSourceFile(self.__TV_SHOW_2_FILE_PROPER_NFO)

        self._createSourceFile(self.__SAMPLE_FILE_1_SOURCE_PATH)
        self._createSourceFile(self.__SAMPLE_FILE_2_SOURCE_PATH)
        self._createSourceFile(self.__SAMPLE_FILE_3_SOURCE_PATH)

        self._createSourceFile(self.__TV_SHOW_5_FILE_SOURCE_PATH)

    def tearDown(self):
        self._delete_test_dirs()

    def test_moving_files(self):
        self.__assert_tv_shows_files([self.__TV_SHOW_2_FILE_1_SOURCE_PATH], [self.__TV_SHOW_2_FILE_1_DESTINATION_PATH])
        self.__assert_tv_shows_files(
            [self.__TV_SHOW_FILE_SOURCE_PATH, self.__TV_SHOW_2_FILE_2_SOURCE_PATH],
            [self.__TV_SHOW_FILE_DESTINATION_PATH, self.__TV_SHOW_2_FILE_2_DESTINATION_PATH])

        # test where season dir already exist
        self.__assert_tv_shows_files(
            [self.__TV_SHOW_3_FILE_SOURCE_PATH],
            [self.__TV_SHOW_3_FILE_DESTINATION_PATH])

        # test that we match against correct dist dir when the file is marked as PROPER
        self.__assert_tv_shows_files(
            [self.__TV_SHOW_4_SOURCE_PATH],
            [self.__TV_SHOW_4_DESTINATION_PATH])

        self.__assert_tv_shows_files(
            [self.__TV_SHOW_5_FILE_SOURCE_PATH],
            [self.__TV_SHOW_5_DESTINATION_PATH]
        )

        self.__assert_tv_shows_files(
            [self.__SAMPLE_FILE_1_SOURCE_PATH, self.__SAMPLE_FILE_2_SOURCE_PATH, self.__SAMPLE_FILE_3_SOURCE_PATH],
            [self.__SAMPLE_FILE_1_DESTINATION_PATH, self.__SAMPLE_FILE_2_DESTINATION_PATH, self.__SAMPLE_FILE_3_DESTINATION_PATH]
        )

    def test_proper_episode_replaces_old(self):
        # moving media flle
        proper_file_source_path = self.__TV_SHOW_2_FILE_PROPER
        proper_file_destination_path = self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_PROPER

        # moving random file with same file name
        proper_file_source_path_nfo = self.__TV_SHOW_2_FILE_PROPER_NFO
        proper_file_destination_path_nfo = self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_PROPER_NFO

        self.__assert_tv_shows_files(
            [proper_file_source_path, proper_file_source_path_nfo],
            [proper_file_destination_path, proper_file_destination_path_nfo]
        )

        # original 1 should not exist anymore
        original_1_path = self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_ORIGINAL_1
        self.assertFalse(file_handler.file_exist(original_1_path))

        original_1_path = self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_ORIGINAL_1_NFO
        self.assertFalse(file_handler.file_exist(original_1_path))

        # original 2 should still exist
        original_2_path = self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_3_ORIGINAL_2
        self.assertTrue(file_handler.file_exist(original_2_path))

        original_2_path = self._SHOW_DESTINATION_DIRECTORY + 'Mordeaa/Season 6/' + self.__TV_SHOW_2_FILE_ORIGINAL_2_NFO
        self.assertTrue(file_handler.file_exist(original_2_path))

    def __assert_tv_shows_files(self, source_paths, destination_paths):
        # first loop and move everything
        for index, source_path in enumerate(source_paths):
            episode_mover.move(self._SHOW_DESTINATION_DIRECTORY, self._SOURCE_DIRECTORY + source_path)

        # run assertions after moving
        for index, source_path in enumerate(source_paths):
            self._assert_file_moved(source_path, destination_paths[index])



from . import episode_mover
from . import file_handler
from . import file_matcher
from . import movie_mover
from . import subtitles
from .media_file_extractor import get_type, Type


def move_media_by_name(name, source_path, show_destination_path, movie_destination_path):
    paths = file_matcher.search_files(name, source_path)

    for path in paths:
        move_media_by_path(path, show_destination_path, movie_destination_path)


def move_media_by_path(path, show_destination_path, movie_destination_path):
    subtitles.rename_and_move(path)
    media_type = get_type(path)
    if media_type is Type.MOVIE:
        movie_mover.move(movie_destination_path, path)
    else:
        episode_mover.move(show_destination_path, path)

    if file_handler.path_is_directory(path):
        file_handler.delete_directory(path)



import unittest

from memover import media_file_extractor
from memover.media_file_extractor import Type
from tests.utils import file_mover_tester


class TestMediaFileExtractor(unittest.TestCase, file_mover_tester.FileMoverTester):

    __TV_SHOWS_PATHS = [
        'testar/test/Another.Another.S02E03.720p.SOMETHING.SOMETHING-SOMETHING.mp4',
        'testar/intetest/Other.Other.Other.Other.S09E01.something.something-something[something].mp4',
        'testar/[ www.flobbing.com ] - Longer.S01E02.HaHV.Maam-A/Information Downloaded From www.akkero.com.txt',
        'testar/www.Lobertar.com - Long.S01E02.720p.KOOK.asdf-Risig/Stuff from www.Lobertar.com.mkv',
        'title/title - Title.S01E02.720p.KOOK.asdf-Risig.mkv',
    ]

    __MOVIE_PATHS = [
        'testar/test/Greater.Great.Greatest.2009.StenRay.123p.DTS.2224-AAA.mkv'
    ]

    def setUp(self):
        self._create_test_dirs()
        for path in self.__TV_SHOWS_PATHS:
            self._createSourceFile(path)

        for path in self.__MOVIE_PATHS:
            self._createSourceFile(path)

    def tearDown(self):
        self._delete_test_dirs()

    def test_type(self):
        self.assertEqual(Type.TV_SHOW, media_file_extractor.get_type(self._SOURCE_DIRECTORY + self.__TV_SHOWS_PATHS[0]))

        self.assertEqual(Type.MOVIE, media_file_extractor.get_type(self._SOURCE_DIRECTORY + self.__MOVIE_PATHS[0]))

        self.assertEqual(Type.TV_SHOW, media_file_extractor.get_type(self._SOURCE_DIRECTORY + self.__TV_SHOWS_PATHS[1]))

    def test_extract_episode(self):
        episode_number = media_file_extractor.get_episode_number(self.__TV_SHOWS_PATHS[0])
        self.assertEqual('03', episode_number)

        self.assertEqual('01', media_file_extractor.get_episode_number(self.__TV_SHOWS_PATHS[1]))

        self.assertRaises(media_file_extractor.WrongMediaTypeException, media_file_extractor.get_season, self.__MOVIE_PATHS[0])

    def test_extract_show_name(self):
        show_name = media_file_extractor.get_tv_show_name(self.__TV_SHOWS_PATHS[2])
        self.assertEqual('Longer', show_name)

    def test_remove_urls_show_name(self):
        show_name = media_file_extractor.get_tv_show_name(self.__TV_SHOWS_PATHS[3])
        self.assertEqual('Long', show_name)

    def test_no_valid_show_name(self):
        self.assertRaises(
            media_file_extractor.CouldNotExtractShowNameException,
            media_file_extractor.get_tv_show_name,
            self.__TV_SHOWS_PATHS[4]
        )

    def test_extract_season(self):
        season_number = media_file_extractor.get_season(self.__TV_SHOWS_PATHS[0])
        self.assertEqual('02', season_number)

        season_number = media_file_extractor.get_season(self.__TV_SHOWS_PATHS[1])
        self.assertEqual('09', season_number)



from . import file_handler, logger


def move(movie_root_path, path):
    if movie_root_path[-1] != '/':
        movie_root_path += '/'

    if file_handler.path_is_directory(path):
        movie_dir = movie_root_path + file_handler.get_last_path_part(path)
        file_handler.create_dir(movie_dir)

        __move_dir(path, movie_dir)
    else:

        movie_dir = movie_root_path + file_handler.get_last_name_from_path(path)
        file_handler.create_dir(movie_dir)

        __move_file(
            path,
            movie_dir + '/' + file_handler.get_last_path_part(path)
        )


def __move_dir(root_source_path, root_destination_path):
    if file_handler.directory_is_empty(root_source_path):
        return

    for relative_source_path in file_handler.get_directory_content(root_source_path):
        source_path = root_source_path + '/' + relative_source_path
        destination_path = root_destination_path + '/' + relative_source_path
        if file_handler.path_is_directory(source_path):
            file_handler.create_dir(destination_path)
            __move_dir(source_path, destination_path)
        else:
            __move_file(
                source_path,
                destination_path
            )


def __move_file(source_path, destination_path):
    file_handler.move(
        source_path,
        destination_path
    )
    logger.info('"' + source_path + '" moved to: "' + destination_path + '"')



import unittest

from memover import file_handler, mover
from tests.utils import file_mover_tester


class TestMoverShows(unittest.TestCase, file_mover_tester.FileMoverTester):

    def setUp(self):
        self._create_test_dirs()

    def tearDown(self):
        self._delete_test_dirs()

    def test_move_show_by_name(self):
        tv_show_file_name = 'All.the.Things.Fire.S02E10.720p.SOMETHING.something-SOMETHING.mkv'
        file_handler.create_file(self._SOURCE_DIRECTORY + tv_show_file_name)

        mover.move_media_by_name(
            'all the things fire',
            self._SOURCE_DIRECTORY,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        destination_path = self._SHOW_DESTINATION_DIRECTORY + '/All the Things Fire/Season 2/' + tv_show_file_name
        file_is_in_new_path = file_handler.file_exist(destination_path)
        self.assertTrue(file_is_in_new_path)

        source_path = self._SOURCE_DIRECTORY + '/' + tv_show_file_name
        file_is_in_old_path = file_handler.file_exist(source_path)
        self.assertFalse(file_is_in_old_path)

    def test_move_show_by_file_path(self):
        tv_show_file_name = 'Man.In.The.on.Folder.S02E03.Dead.AAA.Walking.Wrong-DL.x123.BBA.mp4'
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + tv_show_file_name)

        source_file_path = self._SOURCE_DIRECTORY + '/' + tv_show_file_name
        mover.move_media_by_path(source_file_path, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)

        destination_path = self._SHOW_DESTINATION_DIRECTORY + '/Man In The on Folder/Season 2/' + tv_show_file_name
        self._assert_file_moved(source_file_path, destination_path)

    def test_move_shows_into_existing_season_directory(self):
        file_handler.create_dir(self._SHOW_DESTINATION_DIRECTORY + '/Kontroll US/Season 7/')
        tv_show_file_name = 'kontroll.US.S07E02.720p.AM.SOMETING-OTHER-DEMINSION[aaa]'
        file_source_path = self._SOURCE_DIRECTORY + '/' + tv_show_file_name
        file_handler.create_file(file_source_path)

        # move by name
        mover.move_media_by_name(
            'kontroll',
            self._SOURCE_DIRECTORY,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        file_destination_path = self._SHOW_DESTINATION_DIRECTORY + '/Kontroll US/Season 7/' + tv_show_file_name
        self._assert_file_moved(file_source_path, file_destination_path)

        # move by file
        file_handler.delete_file(file_destination_path)
        file_handler.create_file(file_source_path)
        mover.move_media_by_path(file_source_path, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)
        self._assert_file_moved(file_source_path, file_destination_path)

        # Test with two letter word in the middle of file name that should not be stripped when matching
        file_handler.create_dir(self._SHOW_DESTINATION_DIRECTORY + '/Cool And Welcome To Earth/Season 2/')
        tv_show_file_name = 'Cool.And.Welcome.To.Earth.S02E02.The.Boo.720p.KOL-OL.A300.MP#.mp4'
        file_source_path = self._SOURCE_DIRECTORY + '/' + tv_show_file_name
        file_handler.create_file(file_source_path)

        mover.move_media_by_name(
            'Cool And Welcome To Earth',
            self._SOURCE_DIRECTORY,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        file_destination_path = self._SHOW_DESTINATION_DIRECTORY + '/Cool And Welcome To Earth/Season 2/' + tv_show_file_name
        self._assert_file_moved(file_source_path, file_destination_path)

    def test_moving_episodes_by_file_in_directory(self):
        folder_path = self._SOURCE_DIRECTORY + 'hey.arnold.season1.720p.webdl'
        file_handler.create_dir(folder_path)
        source_file_path_1 = folder_path + '/hey.arnold.S09E01.SOMETHING.something-something'
        file_handler.create_file(source_file_path_1)

        source_file_path_2 = folder_path + '/hey.arnold.S09E02.SOMETHING.something-something.mp4'
        file_handler.create_file(source_file_path_2)

        mover.move_media_by_path(folder_path, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)

        file_destination_path = self._SHOW_DESTINATION_DIRECTORY + '/hey arnold/Season 9/hey.arnold.S09E01.SOMETHING.something-something'
        self._assert_file_moved(source_file_path_1, file_destination_path)

        file_destination_path = self._SHOW_DESTINATION_DIRECTORY + '/hey arnold/Season 9/hey.arnold.S09E02.SOMETHING.something-something.mp4'
        self._assert_file_moved(source_file_path_2, file_destination_path)

    def test_moving_episodes_by_name_in_directory(self):
        folder_path = self._SOURCE_DIRECTORY + '/Cool.And.Welcome.To.Earth.S01.Season.1.720p.5.1Ch.Web-DL.ReEnc-DeeJayAhmed'
        file_handler.create_dir(folder_path)
        file_path_1 = folder_path + '/Cool.And.Welcome.To.Earth.S01E13.720p.5.1Ch.Web-DL.ReEnc-DeeJayAhmed.mkv'
        file_handler.create_file(file_path_1)

        mover.move_media_by_name(
            'Cool And Welcome To Earth',
            self._SOURCE_DIRECTORY,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        self._assert_file_moved(file_path_1, self._SHOW_DESTINATION_DIRECTORY + 'Cool And Welcome To Earth/Season 1/Cool.And.Welcome.To.Earth.S01E13.720p.5.1Ch.Web-DL.ReEnc-DeeJayAhmed.mkv')

    def test_moving_wrong_formatted_episode_inside_right_formated_directory(self):
        folder_path = self._SOURCE_DIRECTORY + '/My.Stuff.Us.On.S10E09.HDTV.CCCC-ARD[axel]'
        source_path = self._createSourceFile(folder_path + '/my.stuff.us.on.1009.fktv-uuu[axel].mkv')

        mover.move_media_by_path(source_path, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)
        self._assert_file_moved(folder_path, self._SHOW_DESTINATION_DIRECTORY + '/My Stuff Us On/Season 10/my.stuff.us.on.1009.fktv-uuu[axel].mkv')

    def test_moving_episode_in_season_dir(self):
        folder_path = 'Kan Inte/Season 3/'
        file_name = 'Kan Inte S03E02 Apptest'
        source_path = self._createSourceFile(folder_path + file_name)
        mover.move_media_by_path(self._SOURCE_DIRECTORY + 'Kan Inte', self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)
        self._assert_file_moved(source_path, self._SHOW_DESTINATION_DIRECTORY + '/' + folder_path + file_name)

    def test_moving_multiple_episodes_with_subtitles_and_instruction_file(self):
        self._createSourceFile('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV.srt')
        self._createSourceFile('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV.mkv')
        self._set_size_in_mb('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV.mkv', 60)

        self._createSourceFile('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV.srt')
        self._createSourceFile('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV.mkv')
        self._set_size_in_mb('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV.mkv', 61)

        self._createSourceFile('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV.srt')
        self._createSourceFile('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV.mkv')
        self._set_size_in_mb('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV.mkv', 62)

        self._createSourceFile('Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Instructions.txt')

        mover.move_media_by_path(self._SOURCE_DIRECTORY + 'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p', self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)

        self._assert_file_moved(
            'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV.srt',
            'show-destination/Kalla Rander, Valla Calla/Season 1/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV.en.srt'
        )

        self._assert_file_moved(
            'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV.mkv',
            'show-destination/Kalla Rander, Valla Calla/Season 1/Kalla Rander, Valla Calla S01E01 (Generationen X) HDTV.mkv'
        )

        self._assert_file_moved(
            'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV.srt',
            'show-destination/Kalla Rander, Valla Calla/Season 1/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV.en.srt'
        )

        self._assert_file_moved(
            'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV.mkv',
            'show-destination/Kalla Rander, Valla Calla/Season 1/Kalla Rander, Valla Calla S01E02 (Generationen X) HDTV.mkv'
        )

        self._assert_file_moved(
            'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV.srt',
            'show-destination/Kalla Rander, Valla Calla/Season 1/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV.en.srt'
        )

        self._assert_file_moved(
            'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV.mkv',
            'show-destination/Kalla Rander, Valla Calla/Season 1/Kalla Rander, Valla Calla S01E03 (Generationen X) HDTV.mkv'
        )

        self._assert_file_moved(
            'Kalla Rander, Valla Calla S01 (Generationen X) HDTV 720p/Instructions.txt',
            'show-destination/Kalla Rander, Valla Calla/Season 1/Instructions.txt'
        )

    def test_moving_show_different_naming_scheme(self):
        episodes = (
            'Last Last - 1x01 - [123].mkv',
            'Last Last - 1x02 - [123].mkv',
            'Last Last - 1x03 - [123].mkv',
            'Last Last - 1x04 - [123].mkv',
            'Last Last - 1x05 - [123].mkv',
            'Last Last - 1x06 - [123].mkv',
            'Last Last - 1x07 - [123].mkv',
            'Last Last - 1x08 - [123].mkv',
            'Last Last - 1x09 - [123].mkv',
            'Last Last - 1x09 - [123].mkv',
            'Last Last - 1x10 - [123].mkv',
            'Last Last - 1x11 - [123].mkv',
            'Last Last - 1x12 - [123].mkv',
            'Last Last - 1x13 - [123].mkv',
            'Last Last - 1x14 - [123].mkv',
            'Last Last - 1x15 - [123].mkv',
            'Last Last - 1x16 - [123].mkv',
            'Last Last - 1x17 - [123].mkv',
            'Last Last - 1x18 - [123].mkv',
            'Last Last - 1x19 - [123].mkv',
            'Last Last - 1x20 - [123].mkv',
            'Last Last - 1x21 - [123].mkv',
            'Last Last - 1x22 - [123].mkv',
            'Last Last - 1x23 - [123].mkv',
            'Last Last - 1x24 - [123].mkv',
            'Last Last - 1x25 - [123].mkv',
            'Last Last - 1x26 - [123].mkv'
        )
        show_name = 'Last Last/'

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

    def test_moving_one_season_only_show(self):
        episodes = (
            '[SyS] Wall Hunger 01 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 02 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 03 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 04 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 05 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 06 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 07 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 08 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 09 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 10 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 11 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 12 (AA 1920x1080 s125 FLAC) [].mkv',
            '[SyS] Wall Hunger 13 (AA 1920x1080 s125 FLAC) [].mkv'
        )

        show_name = '[SyS] Wall Hunger/'

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

        episodes = (
            '[ABA]_and_and_axa_-_01_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_02_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_03_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_04_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_05_-_IjfdsklalllkGirlfriend_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_06_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_07_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_08_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_09_-_Ijfdsklalllk_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_10_-_The_Shosasa_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_11_-_The_Ad_Battle_Contin_[720p]_[ghjk].mkv',
            '[ABA]_and_and_axa_-_12_-_Ijfdsklalllk_[720p]_[ghjk].mkv'
        )

        show_name = '[ABA]_and_and_axa_-/'  # for now show name will be stripped just before episode

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

    def test_moving_one_season_different_episodes_naming_schemes_1(self):
        show_name = 'First Test [ONE_SEASON_DIFFERENT_NAMING_SCHEMES]/'

        episodes = (
            '01 - djksalo-jjjSub].mkv',
            '02 - djksalo-jjjulti-Sub].mkv',
            '03 - djksalo-jjjSub].mkv',
            '04 - djksalo-jjjSub].mkv',
            '05 - djksalo-jjjub].mkv',
            '06 - djksalo-jjjulti-Sub].mkv',
            '07 - djksalo-jjj[Multi-Sub].mkv',
            '08 - djksalo-jjjulti-Sub].mkv',
            '09 - djksalo-jjjudio][Multi-Sub].mkv',
            '10 - djksalo-jjj.mkv'
        )

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

    def test_moving_one_season_different_episodes_naming_schemes_2(self):

        show_name = 'Second Test [ONE_SEASON_DIFFERENT_NAMING_SCHEMES]/'

        episodes = (
            '[ATBC] Jkl Jkl Jkl Jkl Has - 01[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 02[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 03[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 04[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 05[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 06[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 07v2[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 08[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 09[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 10v2[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 11v2[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 12[1233].mkv',
            '[ATBC] Jkl Jkl Jkl Jkl Has - 13[1233].mkv'
        )

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

    def test_moving_one_season_different_episodes_naming_schemes_3(self):

        show_name = 'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES/'

        episodes = (
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 01 (1920x1080 HEVC2 EAC3).mks',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 01 (1920x1080 HEVC2 EAC3).mkv',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 02 (1920x1080 HEVC2 EAC3).mks',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 02 (1920x1080 HEVC2 EAC3).mkv',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 03 (1920x1080 HEVC2 EAC3).mks',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 03 (1920x1080 HEVC2 EAC3).mkv',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 04 (1920x1080 HEVC2 EAC3).mks',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 04 (1920x1080 HEVC2 EAC3).mkv',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 05 (1920x1080 HEVC2 EAC3).mks',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 05 (1920x1080 HEVC2 EAC3).mkv',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 06 (1920x1080 HEVC2 EAC3).mks',
            'Third Test ONE_SEASON_DIFFERENT_NAMING_SCHEMES - 06 (1920x1080 HEVC2 EAC3).mkv'
        )

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

    def test_moving_with_only_season(self):
        episode = 'Lisa.Mirrander.S02.Special.Rocked.Summer.1080p.WEB-Org.AA02.1.LKIO.mkv'
        self._createSourceFile(episode)
        mover.move_media_by_path(self._SOURCE_DIRECTORY + episode, self._SHOW_DESTINATION_DIRECTORY, self._MOVIE_DESTINATION_DIRECTORY)
        self._assert_file_moved(episode, self._SHOW_DESTINATION_DIRECTORY + 'Lisa Mirrander/Season 2/' + episode)

    def test_moving_with_episode_in_file_name(self):
        show_name = 'Al Ber II aew Color/'

        episodes = (
            'Episode 01 - fhdsjk.avi',
            'Episode 02 - The Ale djs.avi',
            'Episode 03 - Alw at as.avi',
            'Episode 04 - hdsk fjdsl dss.avi',
            'Episode 05 - Areda Aa Kanske.avi',
            'Episode 06 - The Felaktig ocd Nord More.avi',
            'Episode 07 - Stong the Ring.avi'
        )

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

    def test_moving_episode_short(self):
        show_name = "Storage Found (1080p A789 10bit dksala)/"

        meta_root_files = (
            'fjdksl a-Alkh AJKL Alkerra JKl Alle akl dajs.website'
            'Ander kl Oker.txt',
            'Jk jk hjkl AHV (jkl Ahtt).txt',
        )

        for file in meta_root_files:
            self._createSourceFile(show_name + file)

        episodes = (
            'Storage Found E02 Ahe jdksle osaf fjdskljkl (1080p A789 10bit dksala).mkv',
            'Storage Found E06 Ass Frdsaosdsaty (1080p A789 10bit dksala).mkv',
            'Storage Found E04 Codsambat Jdasacak (1080p A789 10bit dksala).mkv',
            'Storage Found E07 Bodsambda idsan thdsae Gadasrdsadddden (1080p A789 10bit dksala).mkv',
            'Storage Found E01 Gdsadsae dame (1080p A789 10bit dksala).mkv',
            'Storage Found E05 A dsasaaaa Cat (1080p A789 10bit dksala).mkv',
            'Storage Found E03 Sounding (1080p A789 10bit dksala).mkv'
        )
        self.__create_move_and_validate_season1_episodes(episodes, show_name)

        for file in meta_root_files:
            self._assert_file_moved(show_name + file, self._SHOW_DESTINATION_DIRECTORY + show_name + file)

    def test_same_show_name_on_each_file(self):
        show_source_dir = 'kung fo season 1 (1080p bd 123)/'
        destination_show_path = self._SHOW_DESTINATION_DIRECTORY + '/Kung Fo/Season 1/'

        file_name_1 = 'Kung fo S01E01 Completely Strange (1080p) (C).mkv'
        file_name_2 = '(C) for Audio Commentary.txt'
        file_name_3 = 'Kung fo S01E02 Somewhat Stranger (1080p) (C).mkv'

        file_handler.create_dir(destination_show_path)

        self._createSourceFile(show_source_dir + file_name_1)
        self._createSourceFile(show_source_dir + file_name_2)
        self._createSourceFile(show_source_dir + file_name_3)

        mover.move_media_by_path(
            self._SOURCE_DIRECTORY + show_source_dir,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        self._assert_file_moved(show_source_dir + file_name_1, destination_show_path + file_name_1)
        self._assert_file_moved(show_source_dir + file_name_2, self._SHOW_DESTINATION_DIRECTORY + '/Kung Fo/' + file_name_2)
        self._assert_file_moved(show_source_dir + file_name_3, destination_show_path + file_name_3)

    def test_screens_images_moved_correctly(self):
        show_name = 'what happens in the sun/'
        show_source_dir = '/What Happens in the Sun S01E01 1080p WEB H264'
        source_show_dir = self._SOURCE_DIRECTORY + show_source_dir

        file_handler.create_dir(source_show_dir)
        episode = '/what.happens.in.the.sun.s01e01.1080p.web.h264.mkv'
        file_handler.create_file(source_show_dir + episode)

        screen_dir = '/Screens'
        file_handler.create_dir(source_show_dir + screen_dir)
        screen_file = '/screen0001.jpg'
        file_handler.create_file(source_show_dir + screen_dir + screen_file)

        mover.move_media_by_path(
            source_show_dir,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        self._assert_file_moved(source_show_dir + screen_dir + screen_file, self._SHOW_DESTINATION_DIRECTORY + show_name + screen_file)

        self.__validate_episodes_season_1([episode], show_name)

    def test_multiple_matches(self):
        file_handler.create_dir(f'{self._SHOW_DESTINATION_DIRECTORY}mr test')
        file_handler.create_dir(f'{self._SHOW_DESTINATION_DIRECTORY}Test Test and Test')

        episode_name = 'Mr.Test.S07E01.REPACK.a111-AAA[a]'
        file_handler.create_dir(f'{self._SOURCE_DIRECTORY}{episode_name}')
        file_path = f'{self._SOURCE_DIRECTORY}{episode_name}/{episode_name}.mp4'
        file_handler.create_file(file_path)

        mover.move_media_by_path(
            self._SOURCE_DIRECTORY + episode_name,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )

        self._assert_file_moved(
            file_path,
            f'{self._SHOW_DESTINATION_DIRECTORY}mr test/Season 7/{episode_name}.mp4'
        )

    def test_title_in_file_name(self):
        show_name = 'Mr Wayne A Ale & Faber in The Cure/'

        episodes = [
            'Title 01 - Some title 1.mp4',
            'Title 02 - Some title 2.mp4',
            'Title 03 - Some title 3.mp4',
            'Title 04 - Some title 4.mp4',
            'Title 05 - Some title 5.mp4',
            'Title 06 - Some title 6.mp4'
        ]

        self.__create_move_and_validate_season1_episodes(episodes, show_name)

    def __create_move_and_validate_season1_episodes(self, episodes, show_name):
        for episode in episodes:
            file_path = show_name + episode
            self._createSourceFile(file_path)
            self._set_size_in_mb(file_path, 50)

        mover.move_media_by_path(
            self._SOURCE_DIRECTORY + show_name,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY
        )
        self.__validate_episodes_season_1(episodes, show_name)

    def __validate_episodes_season_1(self, episodes, show_name):
        for episode in episodes:
            self._assert_file_moved(show_name + episode, self._SHOW_DESTINATION_DIRECTORY + show_name + 'Season 1/' + episode)



import asyncio
import logging
import unittest

from memover import logger
from memover.arguments_parser import MeMoverArgs, Commands
from memover.watcher.async_watcher import AsyncWatcher
from tests.utils import file_mover_tester


class TestWatcherModifiedFile(unittest.TestCase, file_mover_tester.FileMoverTester):

    @property
    def test_file_1(self):
        return 'kolla.S04E15.asswe.xTTT-RR[abf].mkv'

    @property
    def test_file_1_path(self):
        return f'{self._SOURCE_DIRECTORY}{self.test_file_1}'

    def setUp(self):
        self._create_test_dirs()

    def tearDown(self):
        self._delete_test_dirs()

    async def make_file_bigger(self):
        # modify and create file
        self._createSourceFile(self.test_file_1)
        self._set_size_in_mb(self.test_file_1, 1)

        await asyncio.sleep(AsyncWatcher.SLEEP_SECONDS)

        self.assertEqual(self.my_watcher.created_paths, [self.test_file_1_path])

        # modify file
        self._set_size_in_mb(self.test_file_1, 1)
        await asyncio.sleep(AsyncWatcher.SLEEP_SECONDS)

        self.assertEqual(self.my_watcher.created_paths, [self.test_file_1_path])

        # modify file
        self._set_size_in_mb(self.test_file_1, 1)

        await asyncio.sleep(AsyncWatcher.SLEEP_SECONDS + 1)

        self.assertEqual(self.my_watcher.created_paths, [self.test_file_1_path])

        # No modification to file.
        # Sleep longer to be sure that file has been moved
        await asyncio.sleep(AsyncWatcher.SLEEP_SECONDS + 1)

        self.assertEqual(self.my_watcher.created_paths, [])

        self._assert_file_moved(
            self.test_file_1,
            f'{self._SHOW_DESTINATION_DIRECTORY}/kolla/Season 4/{self.test_file_1}'
        )

    def test_moving_growing_file(self):
        logger.setup(logging.DEBUG)
        self.my_watcher = None

        args = MeMoverArgs(
            Commands.WATCH,
            self._SOURCE_DIRECTORY,
            self._SHOW_DESTINATION_DIRECTORY,
            self._MOVIE_DESTINATION_DIRECTORY,
            log_level=logging.DEBUG,
            moves_before_quit=1
        )

        self.my_watcher = AsyncWatcher(args, stable_seconds=0)

        async def run_watcher_with_growing_file():
            await asyncio.gather(
                self.my_watcher.observe(),
                self.make_file_bigger()
            )

        asyncio.run(run_watcher_with_growing_file())



import subprocess
import sys
import unittest
from tests.utils import file_mover_tester
from memover import file_handler


class TestApp(unittest.TestCase, file_mover_tester.FileMoverTester):

    __TV_SHOW_FILE_NAME_1 = 'Con.with.fire.Fire.S02E10.720p.SOMETHING.something-SOMETHING.mkv'
    __TV_SHOW_FILE_NAME_2 = 'kolla.S04E15.asswe.xTTT-RR[abf].mkv'
    __TV_SHOW_FILE_NAME_3 = 'Con.with.fire.Fire.S01E01.720p.ANYTHING.ABJ.mkv'
    __MOVIE_FILE_NAME_1 = 'Fenix.Hart.And.Not.2001.DVDRip.DIVX.741-RO.mp4'
    __MOVIE_FILE_NAME_2 = 'ColdCraft (2016) FinSub CDROMRip x264 KORIG'

    __TV_SHOW_FILE_NAME_3_ORIGINAL = 'Old.Stuff.S02E15.720p.HDTV.x264-SVA[rarbg].cold'
    __TV_SHOW_FILE_NAME_3_PROPER = 'Old.Stuff.S02E15.PROPER.720p.HDTV.x264-KILLERS[rarbg].cold'

    __TV_SHOW_FILE_NAME_4 = 'Information Downloaded From www.akkero.com.txt'
    __TV_SHOW_FILE_NAME_4_PARENT_FOLDER = '[ www.gallero.it ] - Longer.S01E02.abg.AQS-A'

    __TV_SHOW_FILE_NAME_5 = 'Stuff from www.Lobertar.com.mkv'
    __TV_SHOW_FILE_NAME_5_PARENT_FOLDER = 'www.Lobertar.com - Long.S01E02.720p.KOOK.asdf-Risig'

    def setUp(self):
        self._create_test_dirs()
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + self.__TV_SHOW_FILE_NAME_1)
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + self.__TV_SHOW_FILE_NAME_2)
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + self.__TV_SHOW_FILE_NAME_3)
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + self.__MOVIE_FILE_NAME_1)
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + self.__MOVIE_FILE_NAME_2)

        file_handler.create_dir(self._SHOW_DESTINATION_DIRECTORY + 'Old Stuff/Season 2/')
        file_handler.create_file(self._SHOW_DESTINATION_DIRECTORY + 'Old Stuff/Season 2/' + self.__TV_SHOW_FILE_NAME_3_ORIGINAL)
        file_handler.create_file(self._SOURCE_DIRECTORY + '/' + self.__TV_SHOW_FILE_NAME_3_PROPER)

        parent_path = self._SOURCE_DIRECTORY + '/' + self.__TV_SHOW_FILE_NAME_4_PARENT_FOLDER
        file_handler.create_dir(parent_path)
        file_handler.create_file(parent_path + '/' + self.__TV_SHOW_FILE_NAME_4)

        parent_path = self._SOURCE_DIRECTORY + '/' + self.__TV_SHOW_FILE_NAME_5_PARENT_FOLDER
        file_handler.create_dir(parent_path)
        file_handler.create_file(parent_path + '/' + self.__TV_SHOW_FILE_NAME_5)

    def tearDown(self):
        self._delete_test_dirs()

    def test_moving_shows_with_wrong_formated_parent_folder(self):
        self.__run_by_file(r'sourcefolder/[\ www.gallero.it\ ]\ -\ Longer.S01E02.abg.AQS-A/Information\ Downloaded\ From\ www.akkero.com.txt')
        destination_path = self._SHOW_DESTINATION_DIRECTORY + 'Longer/Season 1/' + self.__TV_SHOW_FILE_NAME_4
        self._assert_file_moved(
            self._SOURCE_DIRECTORY + self.__TV_SHOW_FILE_NAME_4_PARENT_FOLDER + '/' + self.__TV_SHOW_FILE_NAME_4,
            destination_path
        )

    def test_moving_show_with_urls(self):
        self.__run_by_file(r'sourcefolder/www.Lobertar.com\ -\ Long.S01E02.720p.KOOK.asdf-Risig/Stuff\ from\ www.Lobertar.com.mkv')
        destination_path = self._SHOW_DESTINATION_DIRECTORY + 'Long/Season 1/' + self.__TV_SHOW_FILE_NAME_5
        self._assert_file_moved(
            self._SOURCE_DIRECTORY + self.__TV_SHOW_FILE_NAME_5_PARENT_FOLDER + '/' + self.__TV_SHOW_FILE_NAME_5,
            destination_path
        )

    def test_moving_show_by_episode(self):
        """
        Undocumented but should be possible to move specific episode based by name, season and episode. Used internally
        """
        self.__run_by_name('con with fire fire S01E01')
        destination_path = self._SHOW_DESTINATION_DIRECTORY + 'Con with fire Fire/Season 1/' + self.__TV_SHOW_FILE_NAME_3
        self._assert_file_moved(self._SOURCE_DIRECTORY + '/' + self.__TV_SHOW_FILE_NAME_3, destination_path)

    def test_moving_show_by_name(self):
        self.__run_by_name('con with fire fire')
        destination_path = self._SHOW_DESTINATION_DIRECTORY + 'Con with fire Fire/Season 2/' + self.__TV_SHOW_FILE_NAME_1
        self._assert_file_moved(self._SOURCE_DIRECTORY + self.__TV_SHOW_FILE_NAME_1, destination_path)

    def test_moving_movie_by_name(self):
        self.__run_by_name('ColdCraft')
        destination_path = self._MOVIE_DESTINATION_DIRECTORY + self.__MOVIE_FILE_NAME_2 + '/' + self.__MOVIE_FILE_NAME_2
        self._assert_file_moved(self._SOURCE_DIRECTORY + self.__MOVIE_FILE_NAME_2, destination_path)

    def test_moving_movie_by_file(self):
        self.__run_by_file('sourcefolder/Fenix.Hart.And.Not.2001.DVDRip.DIVX.741-RO.mp4')
        destination_path = self._MOVIE_DESTINATION_DIRECTORY + '/Fenix.Hart.And.Not.2001.DVDRip.DIVX.741-RO/' + self.__MOVIE_FILE_NAME_1
        self._assert_file_moved(self._SOURCE_DIRECTORY + self.__MOVIE_FILE_NAME_1, destination_path)

    def test_moving_show_by_file(self):
        self.__run_by_file('sourcefolder/kolla.S04E15.asswe.xTTT-RR[abf].mkv')
        destination_path = self._SHOW_DESTINATION_DIRECTORY + 'kolla/Season 4/' + self.__TV_SHOW_FILE_NAME_2
        self._assert_file_moved(self._SOURCE_DIRECTORY + self.__TV_SHOW_FILE_NAME_2, destination_path)

    def test_that_proper_replace_old_episode(self):
        self.__run_by_file('sourcefolder/Old.Stuff.S02E15.PROPER.720p.HDTV.x264-KILLERS[rarbg].cold')
        proper_file_destination_path = self._SHOW_DESTINATION_DIRECTORY + 'Old Stuff/Season 2/' + self.__TV_SHOW_FILE_NAME_3_PROPER
        wrong_file_destination_path = self._SHOW_DESTINATION_DIRECTORY + 'Old Stuff/Season 2/' + self.__TV_SHOW_FILE_NAME_3_ORIGINAL
        self.assertFalse(file_handler.file_exist(wrong_file_destination_path))
        self._assert_file_moved(self._SOURCE_DIRECTORY + self.__TV_SHOW_FILE_NAME_3_PROPER, proper_file_destination_path)

    def test_moving_file_by_parent(self):
        source_dir = self._SOURCE_DIRECTORY + self.__TV_SHOW_FILE_NAME_4_PARENT_FOLDER + '/'
        self.__run_by_file(r'sourcefolder/[\ www.gallero.it\ ]\ -\ Longer.S01E02.abg.AQS-A')
        self._assert_file_moved(source_dir, self._SHOW_DESTINATION_DIRECTORY + 'Longer/Season 1/' + self.__TV_SHOW_FILE_NAME_4)

    @staticmethod
    def __run_by_name(show_name):
        TestApp.__run_app(f'by-name "{show_name}" sourcefolder show-destination movie-destination')

    @staticmethod
    def __run_by_file(source_file_path):
        TestApp.__run_app(f'by-path {source_file_path} show-destination movie-destination')

    @staticmethod
    def __run_app(args):
        with subprocess.Popen(
                f'{sys.executable} -m memover {args}',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
        ) as p:
            for line in p.stdout:
                print(line)
            p.wait()



import subprocess
import sys
import time
import unittest

from memover import file_handler
from tests.utils import file_mover_tester


class TestWatcher(unittest.TestCase, file_mover_tester.FileMoverTester):

    __test_file_1 = 'kolla.S04E15.asswe.xTTT-RR[abf].mkv'
    __test_file_2 = 'kolla.S05E16.asswe.xTTT-RR[abf].mkv'

    def setUp(self):
        self._create_test_dirs()

    def tearDown(self):
        self._delete_test_dirs()

    def __get_destination_path_file_1(self):
        return f'{self._SHOW_DESTINATION_DIRECTORY}/kolla/Season 4/{self.__test_file_1}'

    def __get_destination_path_file_2(self):
        return f'{self._SHOW_DESTINATION_DIRECTORY}/kolla/Season 5/{self.__test_file_2}'

    def test_moving_multiple_files(self):
        def run_app():
            args = f'{self._SOURCE_DIRECTORY} {self._SHOW_DESTINATION_DIRECTORY} {self._MOVIE_DESTINATION_DIRECTORY} --moves 2'
            execution = f'{sys.executable} -m memover watch {args}'
            p = subprocess.Popen(execution, shell=True)
            return p

        process = run_app()
        time.sleep(1)  # Run app for a second before creating files

        self._createSourceFile(self.__test_file_1)
        self._createSourceFile(self.__test_file_2)

        process.wait()

        destination_path_file_1 = self.__get_destination_path_file_1()
        file_is_in_new_path = file_handler.file_exist(destination_path_file_1)
        self.assertTrue(file_is_in_new_path)

        file_is_in_new_path = file_handler.file_exist(self.__get_destination_path_file_2())
        self.assertTrue(file_is_in_new_path)



import os
import time

from memover import mover, logger
from memover.arguments_parser import MeMoverArgs
from memover.watcher.path_queue import PathQueue
from pathlib import Path


class SyncedWatcher:
    # Number of seconds a file must be stable (no modifications) before moving
    DEFAULT_STABLE_SECONDS = 5

    def __init__(self, args: MeMoverArgs, stable_seconds: float = None):
        self.__created_paths_queue = PathQueue(args)
        self.__args = args
        self.__stable_seconds = stable_seconds if stable_seconds is not None else self.DEFAULT_STABLE_SECONDS

    def move_next_path(self):
        if self.__created_paths_queue.is_empty():
            return False

        queue_item = self.__created_paths_queue.next()

        # First check: if file is in modified,set it to false and wait for next cycle
        if queue_item.modified:
            queue_item.modified = False
            logger.debug(f'File {queue_item.path} was modified recently, waiting for next cycle')
            return False

        # Second check: ensure file was stable for stable_seconds after last modification
        time_since_last_modify = self.__created_paths_queue.time_since_last_modified(queue_item.path)
        if time_since_last_modify is not None and time_since_last_modify < self.__stable_seconds:
            logger.debug(f'File {queue_item.path} was modified {time_since_last_modify:.1f}s ago, waiting for {self.__stable_seconds}s of stability')
            return False

        self.__log_queue_status('Queues Before Move: Before removing path')

        self.__created_paths_queue.remove(queue_item.path)

        self.__log_queue_status('Queues Before Move: after removing path')
        mover.move_media_by_path(
            queue_item.path,
            self.__args.show_destination,
            self.__args.movie_destination
        )
        self.__log_queue_status('Queues After Move')

        return True

    def __log_queue_status(self, header):
        logger.debug(f'-- {header} --')
        self.__created_paths_queue.log_debug('created_paths')
        logger.debug('-- End --')

    def __is_source_dir(self, path):
        return path.strip('/') == self.__args.source.strip('/') or Path(self.__args.source).parent == Path(path)

    @property
    def created_paths(self):
        return self.__created_paths_queue

    def on_created(self, path):
        if self.__is_source_dir(path):
            return

        if not os.path.exists(path):
            logger.debug(f'FileCreated event for path {path} that does not exist')
            return

        self.__created_paths_queue.append(path, time.time())

        self.on_modified(path)

    def on_modified(self, path):
        if self.__is_source_dir(path):
            return

        if not os.path.exists(path):
            return

        if not self.__created_paths_queue.in_queue(path):
            return

        # Track the time of this modification
        current_time = time.time()
        self.__created_paths_queue.update_last_modified(path, current_time)



import asyncio
import time

from watchdog.events import PatternMatchingEventHandler
from watchdog.observers import Observer

from memover import logger
from memover.arguments_parser import MeMoverArgs
from memover.watcher.synced_watcher import SyncedWatcher


class AsyncWatcher:

    SLEEP_SECONDS = 2

    def __init__(self, args: MeMoverArgs, stable_seconds: float = None) -> None:
        self.__start_time = time.time()
        self.__args = args
        self.__synced_watcher = SyncedWatcher(args, stable_seconds=stable_seconds)
        self.__moves = 0

    @property
    def created_paths(self):
        return self.__synced_watcher.created_paths

    def move_created_files(self):
        success = self.__synced_watcher.move_next_path()
        if success:
            self.__moves += 1

    def on_created(self, event):
        self.__synced_watcher.on_created(event.src_path)

    def on_modified(self, event):
        self.__synced_watcher.on_modified(event.src_path)

    async def observe(self):
        my_event_handler = PatternMatchingEventHandler(ignore_directories=False, case_sensitive=True)
        my_event_handler.on_created = self.on_created
        my_event_handler.on_modified = self.on_modified

        path = self.__args.source
        logger.info(f"Watching: {path}")
        go_recursively = True
        my_observer = Observer()
        my_observer.schedule(my_event_handler, path, recursive=go_recursively)
        my_observer.start()

        try:
            while not self.should_quit():
                self.move_created_files()
                await asyncio.sleep(self.SLEEP_SECONDS)
        finally:
            my_observer.stop()
            my_observer.join()

    def should_quit(self):
        if self.__args.moves_before_quit and self.__moves >= self.__args.moves_before_quit:
            return True

        return False


async def run(args: MeMoverArgs):
    my_watcher = AsyncWatcher(args)
    await asyncio.gather(
        my_watcher.observe()
    )
